//
//  UVInitialLoadManager.m
//  UserVoice
//
//  Created by Austin Taylor on 12/10/12.
//  Copyright (c) 2012 UserVoice Inc. All rights reserved.
//

#import "UVInitialLoadManager.h"
#import "UVHelpTopic.h"
#import "UVArticle.h"
#import "UVAccessToken.h"
#import "UVRequestToken.h"
#import "UVClientConfig.h"
#import "UVConfig.h"
#import "UVUser.h"
#import "UVSession.h"
#import "UVRequestContext.h"
#import "UVUtils.h"
#import "UVForum.h"
#import "UVBabayaga.h"

@implementation UVInitialLoadManager {
    UIAlertView *_errorAlertView;
}

@synthesize dismissed;

+ (UVInitialLoadManager *)loadWithDelegate:(id)delegate action:(SEL)action {
    UVInitialLoadManager *manager = [[UVInitialLoadManager alloc] initWithDelegate:delegate action:action];
    [manager beginLoad];
    return manager;
}

- (id)initWithDelegate:(id)theDelegate action:(SEL)theAction {
    if (self = [super init]) {
        delegate = theDelegate;
        action = theAction;
    }
    return self;
}

- (void)beginLoad {
    if ([UVSession currentSession].clientConfig) {
        [self didLoadClientConfig];
    } else {
        [UVClientConfig getWithDelegate:self];
    }
    [self loadUser];
}

- (void)loadUser {
    if ([UVSession currentSession].config.ssoToken != nil || [UVSession currentSession].config.email != nil) {
        [UVRequestToken getRequestTokenWithDelegate:self];
    } else if ([UVAccessToken exists]) {
        [UVSession currentSession].accessToken = [[[UVAccessToken alloc] initWithExisting] autorelease];
        [UVUser retrieveCurrentUser:self];
    } else {
        userDone = YES;
    }
    [self checkComplete];
}

- (void)checkComplete {
    if (configDone && userDone && topicsDone && articlesDone && forumDone) {
        if ([UVSession currentSession].user) {
            [[UVSession currentSession].user updateVotesRemaining];
        }
        if (dismissed) return;
        [delegate performSelector:action];
    }
}

- (void)didRetrieveRequestToken:(UVRequestToken *)token {
    if (dismissed) return;
    [UVSession currentSession].requestToken = token;
    if ([UVSession currentSession].config.ssoToken != nil) {
        [UVUser findOrCreateWithSsoToken:[UVSession currentSession].config.ssoToken delegate:self];
    } else if ([UVSession currentSession].config.email != nil) {
        [UVUser findOrCreateWithGUID:[UVSession currentSession].config.guid andEmail:[UVSession currentSession].config.email andName:[UVSession currentSession].config.displayName andDelegate:self];
    }
}

- (void)didRetrieveClientConfig:(UVClientConfig *)clientConfig {
    if (dismissed) return;
    [UVSession currentSession].clientConfig = clientConfig;
    [self didLoadClientConfig];
}

- (void)didLoadClientConfig {
    UVClientConfig *clientConfig = [UVSession currentSession].clientConfig;
    configDone = YES;
    if (clientConfig.ticketsEnabled) {
        if ([UVSession currentSession].config.topicId) {
            [UVHelpTopic getTopicWithId:[UVSession currentSession].config.topicId delegate:self];
            [UVArticle getArticlesWithTopicId:[UVSession currentSession].config.topicId delegate:self];
        } else {
            [UVHelpTopic getAllWithDelegate:self];
            [UVArticle getArticlesWithDelegate:self];
        }
    } else {
        topicsDone = YES;
        articlesDone = YES;
    }
    if (clientConfig.feedbackEnabled) {
        [UVForum getWithId:[UVSession currentSession].config.forumId delegate:self];
    } else {
        forumDone = YES;
    }
    [self checkComplete];
}

- (void)didRetrieveForum:(UVForum *)forum {
    if (dismissed) return;
    [UVSession currentSession].forum = forum;
    forumDone = YES;
    [self checkComplete];
}

- (void)didCreateUser:(UVUser *)theUser {
    if (dismissed) return;
    [UVSession currentSession].user = theUser;
    [[UVSession currentSession].accessToken persist];
    [UVBabayaga track:IDENTIFY];
    userDone = YES;
    [self checkComplete];
}

- (void)didRetrieveCurrentUser:(UVUser *)theUser {
    if (dismissed) return;
    [UVSession currentSession].user = theUser;
    [[UVSession currentSession].accessToken persist];
    userDone = YES;
    [self checkComplete];
}

- (void)didRetrieveHelpTopic:(UVHelpTopic *)topic {
    if (dismissed) return;
    [UVSession currentSession].topics = @[topic];
    topicsDone = YES;
    [self checkComplete];
}

- (void)didRetrieveHelpTopics:(NSArray *)topics {
    if (dismissed) return;
    [UVSession currentSession].topics = [topics filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:@"articleCount > 0"]];
    topicsDone = YES;
    [self checkComplete];
}

- (void)didRetrieveArticles:(NSArray *)articles {
    if (dismissed) return;
    [UVSession currentSession].articles = articles;
    articlesDone = YES;
    [self checkComplete];
}

- (void)alertView:(UIAlertView *)alertView didDismissWithButtonIndex:(NSInteger)buttonIndex {
    [delegate performSelector:@selector(dismissUserVoice)];
}

- (void)didReceiveError:(NSError *)error context:(UVRequestContext *)requestContext {
    if (dismissed) return;
    NSString *message = nil;
    if ([UVUtils isAuthError:error]) {
        if ([requestContext.context isEqualToString:@"sso"] || [requestContext.context isEqualToString:@"local-sso"]) {
          // SSO and local SSO can fail with regard to admins. It's ok to proceed without a user.
          userDone = YES;
          return;
        }
        if ([UVAccessToken exists]) {
            [[UVSession currentSession].accessToken remove];
            [UVSession currentSession].accessToken = nil;
            [self loadUser];
            return;
        } else {
            message = NSLocalizedStringFromTable(@"This application didn't configure UserVoice properly", @"UserVoice", nil);
        }
    } else if ([UVUtils isConnectionError:error]) {
        message = NSLocalizedStringFromTable(@"There appears to be a problem with your network connection, please check your connectivity and try again.", @"UserVoice", nil);
    } else {
        message = NSLocalizedStringFromTable(@"Sorry, there was an error in the application.", @"UserVoice", nil);
    }
    
    if (_errorAlertView) {
        return;
    }
    
    _errorAlertView = [[UIAlertView alloc] initWithTitle:NSLocalizedStringFromTable(@"Error", @"UserVoice", nil)
                                                 message:message
                                                delegate:self
                                       cancelButtonTitle:nil
                                       otherButtonTitles:NSLocalizedStringFromTable(@"OK", @"UserVoice", nil), nil];
    [_errorAlertView autorelease];
    [_errorAlertView show];
}

- (void)dealloc {
    if (_errorAlertView) {
        _errorAlertView.delegate = nil;
    }
    
    [super dealloc];
}


@end
